local AzerPUG_RareUtility, RareUtil = ...

local L = LibStub("AceLocale-3.0"):GetLocale("AzerPUGRareUtility", false)

local AnnounceZone = nil
local lastTargetID = nil
local UpdateInterval = 1.0
local raresInfo = RareUtil.raresInfo

local addon = LibStub("AceAddon-3.0"):NewAddon("RareUtil", "AceConsole-3.0")
local RareUtilLDB = LibStub("LibDataBroker-1.1"):NewDataObject("RareUtil", {
	type = "data source",
	text = "RareUtil",
	icon = "Interface\\Icons\\Inv_darkmoon_eye",
	OnClick = function() RareUtil_AddonFrameToggle() end
})
local icon = LibStub("LibDBIcon-1.0")

local dash = " - "
local promo = "AzerPUG RareUtility"

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RareUtilDB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("RareUtil", RareUtilLDB, self.db.profile.minimap)
	self:RegisterChatCommand("RareUtil icon", "RareUtilMiniMapIconToggle")
end

function addon:RareUtilMiniMapIconToggle()
	self.db.profile.minimap.hide = not self.db.profile.minimap.hide
	if self.db.profile.minimap.hide then
		icon:Hide("RareUtil")
	else
		icon:Show("RareUtil")
	end
end

function RareUtil_OnLoad(self)
    local RareUtilAddonFrame = CreateFrame("FRAME", "RareUtilAddonFrame", UIParent)
    RareUtilAddonFrame:SetPoint("CENTER", 0, 0)
    RareUtilAddonFrame.texture = RareUtilAddonFrame:CreateTexture()
    RareUtilAddonFrame.texture:SetAllPoints(true)
    RareUtilAddonFrame:EnableMouse(true)
    RareUtilAddonFrame:SetMovable(true)
    RareUtilAddonFrame:SetScript("OnEvent", RareUtil__EventHandler)
    RareUtilAddonFrame:RegisterForDrag("LeftButton")
    RareUtilAddonFrame:SetScript("OnDragStart", RareUtilAddonFrame.StartMoving)
    RareUtilAddonFrame:SetScript("OnDragStop", RareUtilAddonFrame.StopMovingOrSizing)
    RareUtilAddonFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    RareUtilAddonFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
    RareUtilAddonFrame:RegisterEvent("ADDON_LOADED")
    RareUtilAddonFrame:RegisterEvent("PLAYER_LOGIN")
    RareUtilAddonFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    RareUtilAddonFrame.TimeSinceLastUpdate = 0

    local frameHeight = (((#raresInfo[GetZoneText()] / 2) * 12) + 45)
    RareUtilAddonFrame:SetSize(250, frameHeight)

    local AddonTitle = RareUtilAddonFrame:CreateFontString("AddonTitle", "ARTWORK", "GameFontNormal")
    AddonTitle:SetText("AzerPUG RareUtility")
    AddonTitle:SetWidth("100")
    AddonTitle:SetHeight("10")
    AddonTitle:SetPoint("TOP", "RareUtilAddonFrame", -55, -3)

    local ZoneSubHeader = RareUtilAddonFrame:CreateFontString("ZoneSubHeader", "ARTWORK", "GameFontNormalSmall")
    ZoneSubHeader:SetText(" - UNSPECIFIED - ")
    ZoneSubHeader:SetWidth("100")
    ZoneSubHeader:SetHeight("10")
    ZoneSubHeader:SetPoint("TOP", "RareUtilAddonFrame", -55, -15) 

    zone = GetZoneText()
    ZoneSubHeader:SetText(zone)

    local OptionsPanel = CreateFrame("FRAME", "AzerPUG RareUtility")
    OptionsPanel.name = "AzerPUG RareUtility"
    InterfaceOptions_AddCategory(OptionsPanel)

    local OptionsHeader = OptionsPanel:CreateFontString("OptionsHeader", "ARTWORK", "GameFontNormalHuge")
    OptionsHeader:SetText(promo .. dash .. L["Options"])
    OptionsHeader:SetWidth(OptionsPanel:GetWidth())
    OptionsHeader:SetHeight(OptionsPanel:GetHeight())
    OptionsHeader:SetPoint("TOP", 0, -10) 

    local AutomaticOptionsCheckButton = CreateFrame("CheckButton", "AutomaticOptionsCheckButton", OptionsPanel, "ChatConfigCheckButtonTemplate")
    AutomaticOptionsCheckButton:SetPoint("TOPLEFT", 25, -50)
    AutomaticOptionsCheckButtonText:SetText(L[" Automatic Announcements"])
    AutomaticOptionsCheckButton.tooltip = L["Toggles the Automatic Announcement function on or off."]
    AutomaticOptionsCheckButton:SetScript("OnClick",
    function()
        if AutomaticOptionsCheckButton:GetChecked() == true then
            RareUtil_ToggleAuto()
        elseif AutomaticOptionsCheckButton:GetChecked() == false then
            RareUtil_ToggleAuto()
        end
    end);

    local AutomaticDeathAnnounceCheckButton = CreateFrame("CheckButton", "AutomaticDeathAnnounceCheckButton", OptionsPanel, "ChatConfigCheckButtonTemplate")
    AutomaticDeathAnnounceCheckButton:SetPoint("TOPLEFT", 25, -75)
    AutomaticDeathAnnounceCheckButtonText:SetText(L[" Automatic Death Announcements"])
    AutomaticDeathAnnounceCheckButton.tooltip = L["Toggles the Automatic Death Announcement function on or off."]
    AutomaticDeathAnnounceCheckButton:SetScript("OnClick",
    function()
        if AutomaticDeathAnnounceCheckButton:GetChecked() == true then
            RareUtil_AutoDeathAnnounce = true
            ChatFrame1:AddMessage(L["Automatic Death Announcements Enabled!"])
        elseif AutomaticDeathAnnounceCheckButton:GetChecked() == false then
            RareUtil_AutoDeathAnnounce = false
            ChatFrame1:AddMessage(L["Automatic Death Announcements Disabled!"])
        end
    end);

    local NazjaRemindersCheckButton = CreateFrame("CheckButton", "NazjaRemindersCheckButton", OptionsPanel, "ChatConfigCheckButtonTemplate")
    NazjaRemindersCheckButton:SetPoint("TOPLEFT", 25, -100)
    NazjaRemindersCheckButtonText:SetText(L[" Commanders/Summons automatic reminders (Nazjatar)"])
    NazjaRemindersCheckButton.tooltip = L["Toggles the Automatic Reminders for Nazjatar Commanders and Summons of the Depths on or off."]
    NazjaRemindersCheckButton:SetScript("OnClick",
    function()
        if NazjaRemindersCheckButton:GetChecked() == true then
            RareUtil_AutoNazjarReminders = true
            RareUtilAddonFrame:SetScript("OnUpdate", RareUtil_OnUpdate)
        elseif NazjaRemindersCheckButton:GetChecked() == false then
            RareUtil_AutoNazjarReminders = false
            RareUtilAddonFrame:SetScript("OnUpdate", nil)
        end
    end);

    local RareUtilityAnnounceButton = CreateFrame("Button", "RareUtilityAnnounceButton", RareUtilAddonFrame, "UIPanelButtonTemplate")
    RareUtilityAnnounceButton.contentText = RareUtilityAnnounceButton:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
    RareUtilityAnnounceButton.contentText:SetText("Announce Rare")
    RareUtilityAnnounceButton:SetWidth("100")
    RareUtilityAnnounceButton:SetHeight("25")
    RareUtilityAnnounceButton.contentText:SetWidth("100")
    RareUtilityAnnounceButton.contentText:SetHeight("15")
    RareUtilityAnnounceButton:SetPoint("TOP", 55, -5)
    RareUtilityAnnounceButton.contentText:SetPoint("CENTER", 0, -1)
    RareUtilityAnnounceButton:SetScript("OnClick", function() RareUtil_CheckRareMob(true) end )
end

function RareUtil_OnUpdate(self, elapsed)
    self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed
    while (self.TimeSinceLastUpdate > UpdateInterval) do
        local curH, curM = GetGameTime()
        
        if curH % 3 == 0 and curM == 0 then    
            self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - 60.0
            RareUtil_AutoReminderMessage("com", 0)
        elseif curH % 3 == 0 and curM == 45 then
            self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - 60.0
            RareUtil_AutoReminderMessage("sum", 15)
        elseif curH % 3 == 0 and curM == 55 then
            self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - 60.0
            RareUtil_AutoReminderMessage("sum", 5)
        elseif curH % 3 == 1 and curM == 0 then 
            self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - 60.0
            RareUtil_AutoReminderMessage("sum", 0)
        elseif curH % 3 == 2 and curM == 45 then
            self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - 60.0
            RareUtil_AutoReminderMessage("com", 15)
        elseif curH % 3 == 2 and curM == 55 then
            self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - 60.0
            RareUtil_AutoReminderMessage("com", 5)
        end
        self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - UpdateInterval
    end
end

function RareUtil_CreateUnitButtons(zone)
    for i = 1, #raresInfo[zone] do
        local raresZI = raresInfo[zone][i]
        if raresZI[5] == nil then
            raresZI[5] = CreateFrame("Button", nil, RareUtilAddonFrame)
            raresZI[5].contentText = raresZI[5]:CreateFontString(nil, "ARTWORK", "GameFontNormal")
            raresZI[5].contentText:SetText(raresZI[2])
            raresZI[5]:SetWidth("104")
            raresZI[5]:SetHeight("10")
            if i <= math.ceil(#raresInfo[zone] / 2) then
                raresZI[5]:SetPoint("TOP", -65, (-12 * i - 30))
                raresZI[5].contentText:SetPoint("LEFT", 0, 0)
            else
                raresZI[5]:SetPoint("TOP", 65, (-12 * (i-math.ceil(#raresInfo[zone] / 2)) -30))
                raresZI[5].contentText:SetPoint("LEFT", 0, 0)
            end
            local xLocation = raresZI[11][1][1]
            local yLocation = raresZI[11][1][2]
            local name = raresZI[2]
            raresZI[5]:SetScript("OnClick", function() 
                if xLocation == "??" or yLocation == "??" then
                    print(L["Rare has more then one spawn location, implementation coming soon!"])
                else
                    TomTom:AddWaypointToCurrentZone(xLocation, yLocation, name) 
                end
            end)

            local LootKnown = RareUtil_AllLootKnown(zone, raresZI[1])

            raresZI[5].contentText:SetTextColor(RareUtil_ButtonColors(zone, raresZI[1]))
            
        end
    end
end

function RareUtil_SavedVarsLoad()
    if RareUtil_AutomaticAnnounce == true then
        AutomaticOptionsCheckButton:SetChecked(true)
    else
        AutomaticOptionsCheckButton:SetChecked(false)
    end
    if RareUtil_AutoNazjarReminders == true then
        NazjaRemindersCheckButton:SetChecked(true)
    else
        NazjaRemindersCheckButton:SetChecked(false)
    end
    if RareUtil_AutoDeathAnnounce == true then
        AutomaticDeathAnnounceCheckButton:SetChecked(true)
    else
        AutomaticDeathAnnounceCheckButton:SetChecked(false)
    end

    RareUtilAddonFrame.texture:SetColorTexture(0.5, 0.5, 0.5, 0.5)
end

function RareUtil__EventHandler(self, event, ...)
    local zone =  GetZoneText()
    if event == "PLAYER_LOGIN" then
        RareUtil_SavedVarsLoad()
        for iterationZone, rares in pairs(raresInfo) do
            RareUtil_CreateUnitButtons(iterationZone)
        end
        RareUtil_ShowHideFrame(zone)
    elseif event == "ZONE_CHANGED_NEW_AREA" then
        RareUtil_ShowHideFrame(zone)
    elseif event == "PLAYER_TARGET_CHANGED" then
        for iterationZone, rares in pairs(raresInfo) do
            if zone == iterationZone then
                for i = 1, #rares do
                    if lastTargetID == rares[i][1] then
                        rares[i][5].contentText:SetTextColor(RareUtil_ButtonColors(zone, rares[i][1]))
                    elseif lastTargetID == 152569 then
                        rares[7][5].contentText:SetTextColor(RareUtil_ButtonColors(zone, 152569, 55812))
                    elseif lastTargetID == 152570 then
                        rares[7][5].contentText:SetTextColor(RareUtil_ButtonColors(zone, 152570, 55812))
                    elseif lastTargetID == 154342 then
                        rares[1][5].contentText:SetTextColor(RareUtil_ButtonColors(zone, 154342, 55512))
                    elseif lastTargetID == 154968 then
                        rares[2][5].contentText:SetTextColor(RareUtil_ButtonColors(zone, 154968, 55546))
                    end
                end
                if RareUtil_AutomaticAnnounce == true then
                    RareUtil_CheckRareMob(false)
                end
            end
        end
    elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
        local v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11 = CombatLogGetCurrentEventInfo()
        if v2 == "UNIT_DIED" then
            if RareUtil_AutoDeathAnnounce == true then
                local UnitID = tonumber(RareUtil_GetUnitIDFromGUID(v8))
                for iterationZone, rares in pairs(raresInfo) do
                    for i = 1, #rares do
                        local announceChannel = RareUtil_CheckGeneralChat()
                        if UnitID == rares[i][1] then
                            if rares[i][3] == true then
                                local spawnTimer = RareUtil_GetRespawnTimer(GetZoneText(), rares[i][1])
                                if spawnTimer == nil then
                                    SendChatMessage((rares[i][2] .. L[" dead!"] ..  dash .. promo), "CHANNEL", nil, announceChannel)
                                else
                                    SendChatMessage((rares[i][2] .. L[" dead!"] .. dash .. L["Expected respawn: "] .. spawnTimer .. dash .. promo), "CHANNEL", nil, announceChannel)
                                end
                                rares[i][4] = time()
                                rares[i][3] = false
                            end
                        elseif UnitID == 149847 or UnitID == 152569 or UnitID == 152570 then
                            SendChatMessage((rares[7][2] .. L[" dead!"] ..  dash .. promo), "CHANNEL", nil, announceChannel)
                        elseif lastTargetID == 154342 then
                            SendChatMessage((rares[1][2] .. L[" (AltTime) dead!"] ..  dash .. promo), "CHANNEL", nil, announceChannel)
                        elseif lastTargetID == 154968 then
                            SendChatMessage((rares[2][2] .. L[" (AltTime) dead!"] ..  dash .. promo), "CHANNEL", nil, announceChannel)
                        end
                    end
                end
            end
        end
    end
end

function RareUtil_ButtonColors(zone, UnitID, trackInput)
    local LootKnown = RareUtil_AllLootKnown(zone, UnitID)
    for i = 1, #raresInfo[zone] do
        local rareZI = raresInfo[zone][i]
        local trackID = trackInput or rareZI[7]
        if UnitID == rareZI[1] then
            if rareZI[7] ~= nil then
                local QuestCompleted = IsQuestFlaggedCompleted(rareZI[7])
                if QuestCompleted == true and LootKnown == true then 
                    return 0, 1, 0, 1
                elseif QuestCompleted == true and LootKnown == false then 
                    return 1, 1, 0, 1
                elseif QuestCompleted == false and LootKnown == true then 
                    return 0, 1, 1, 1
                elseif QuestCompleted == false and LootKnown == false then 
                    return 1, 0, 0, 1
                end
            else
                return 0, 0, 0, 1.0
            end
        end
    end
end

function RareUtil_AutoReminderMessage(textEvent, minLeft)
    local reminder = L["Spawn Reminder!"]
    local eventText = nil
    local time = nil
    local announceChannel = RareUtil_CheckGeneralChat()
    local zone = GetZoneText()

    if textEvent == "com" then
        eventText = L["Azshara's Elite Commanders"]
    elseif textEvent == "sum" then
        eventText = L["Summons from the Depths"]
    end

    if minLeft == 15 then
        time = L["about 15 minutes!"]
    elseif minLeft == 5 then
        time = L["about 5 minutes!"]
    elseif minLeft == 0 then
        time = L["any moment!"]
    end

    local message = (reminder .. dash .. eventText .. dash .. time .. dash .. promo)
    if zone == "Nazjatar" then
        SendChatMessage(message, "CHANNEL", nil, announceChannel)
    end
end

function RareUtil_ShowHideFrame(zone)
    ZoneSubHeader:SetText(zone)
    if raresInfo[zone] ~= nil then
        RareUtilAddonFrame:Show()
    else
        RareUtilAddonFrame:Hide()
    end
    RareUtil_ShowHideButtons(zone)
end

function RareUtil_ShowHideButtons(zone)
    for selectedZone, rares in pairs(raresInfo) do
        for i = 1, #rares do
            rares[i][5]:Hide()
            if zone == selectedZone then
                rares[i][5]:Show()
            end
        end
    end
end

function RareUtil_AllLootKnown(zone, UnitID)
    if RareUtil_IsPetKnown(zone, UnitID) == true and RareUtil_IsMountKnown(zone, UnitID) == true and RareUtil_IsToyKnown(zone, UnitID) == true then
        return true
    else
        return false
    end
end

function RareUtil_IsMountKnown(zone, UnitID)
    local raresZ = raresInfo[zone]
    for i = 1, #raresZ do
        rareZI = raresZ[i]
        if UnitID == rareZI[1] then
            if rareZI[12] ~= nil then
                local allMounts = C_MountJournal.GetMountIDs()
                for i = 1, #allMounts do
                    local currentMountLooped = {C_MountJournal.GetMountInfoByID(allMounts[i])}
                    if currentMountLooped[2] == rareZI[12] then
                        if currentMountLooped[11] == true then
                            return true
                        elseif currentMountLooped[11] == false then
                            return false
                        end
                    end
                end
            else
                return true
            end
        end
    end
end

function RareUtil_IsPetKnown(zone, UnitID)
    local raresZ = raresInfo[zone]
    for i = 1, #raresZ do
        rareZI = raresZ[i]
        if UnitID == rareZI[1] then
            if rareZI[13] ~= nil then
                for petIndex = 1, 1248 do
                    local _, _, owned, _, _, _, _, name, _, _, companionID, tooltip, _, _, _, _, _, _ = C_PetJournal.GetPetInfoByIndex(petIndex)
                    if companionID == rareZI[13] then
                        if owned == true then
                            return true
                        elseif owned == false then
                            return false
                        end
                    end
                end
            else
                return true
            end
        end
    end
end

function RareUtil_IsToyKnown(zone, UnitID)
    local raresZ = raresInfo[zone]
    for i = 1, #raresZ do
        rareZI = raresZ[i]
        if UnitID == rareZI[1] then
            if rareZI[14] ~= nil then
                if PlayerHasToy(rareZI[14]) == true then
                    return true
                elseif PlayerHasToy(rareZI[14]) == false then
                    return false
                end
            else
                return true
            end
        end
    end
end

function RareUtil_CheckGeneralChat()
    local channelName = EnumerateServerChannels()
    local zone = GetZoneText()
    local channelID = GetChannelName(channelName .. " - " .. zone)
    return channelID
end

function RareUtil_CheckRareMob(buttonClicked)
    if UnitExists("Target") then
        local name = UnitName("target")
        local coords = RareUtil_GetPlayerCoords()
        local unitHealth = RareUtil_GetTargetHealth()
        local zone = GetZoneText()
        local UnitID = RareUtil_GetUnitIDFromGUID(UnitGUID("target"))
        local respawnTimer = RareUtil_GetRespawnTimer(zone, UnitID)

        if UnitID == 149847 then
            name = (name .. L[" (Orange)"])
            RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
            raresInfo["Mechagon"][7][4] = time()
        elseif UnitID == 152569 then
            name = (name .. L[" (Green)"])
            RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
            raresInfo["Mechagon"][7][4] = time()
        elseif UnitID == 152570 then
            name = (name .. L[" (Blue)"])
            RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
            raresInfo["Mechagon"][7][4] = time()
        elseif lastTargetID == 154342 then
            name = (name .. L[" (AltTime)"])
            RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
            raresInfo["Mechagon"][1][4] = time()
        elseif lastTargetID == 154968 then
            name = (name .. L[" (AltTime)"])
            RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
            raresInfo["Mechagon"][2][4] = time()
        elseif lastTargetID == 151623 then
            name = (name .. L[" (Mount)"])
            RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
            raresInfo["Mechagon"][35][4] = time()
        else
            for i = 1, #raresInfo[zone] do
                if UnitID == raresInfo[zone][i][1] then
                    if UnitID == 151672 then
                        for i = 1, 40 do
                            local v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15 = UnitAura("player", i, nil) 
                            if v11 == 296644 then
                                name = (name .. L[" (AltTime)"])
                            end
                        end
                    end
                    if buttonClicked == true then
                        RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
                        raresInfo[zone][i][4] = time()
                        raresInfo[zone][i][3] = true
                    else
                        if buttonClicked == false then
                            if raresInfo[zone][i][4] < (time() - 300) then
                                RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
                                raresInfo[zone][i][4] = time()
                                raresInfo[zone][i][3] = true
                            end
                        end
                    end
                end
            end
        end

        if name == "Murloco" then
            RareUtil_AnnounceMurloco()
        end

        if name == "RAG-TAG" then
            RareUtil_AnnounceArmory(coords)
        end
        lastTargetID = UnitID
    else
        lastTargetID = nil
    end
end

function RareUtil_AnnounceRare(name, coords, unitHealth, respawnTimer)
    local percent = "%"
    local life = L[" life."]
    local announceChannel = RareUtil_CheckGeneralChat()

    if unitHealth == 100 then
        SendChatMessage((name .. dash .. coords .. dash .. L["Unengaged!"] .. dash .. promo), "CHANNEL", nil, announceChannel)
    elseif unitHealth > 5 then
        SendChatMessage((name .. dash .. coords .. dash .. unitHealth .. percent .. life .. dash .. promo), "CHANNEL", nil, announceChannel)
    end
end 

function RareUtil_AnnounceArmory(coords)
    local announceChannel = RareUtil_CheckGeneralChat()
    SendChatMessage((L["Completed Armory"] .. dash .. coords .. dash .. promo), "CHANNEL", nil, announceChannel)
end

function RareUtil_AnnounceMurloco()
    local announceChannel = RareUtil_CheckGeneralChat()
    SendChatMessage((L["Murloco found!"] .. dash .. "46 32" .. dash .. promo), "CHANNEL", nil, announceChannel)
end

function RareUtil_GetPlayerCoords()
    loc = C_Map.GetBestMapForUnit("player")
    local pos = C_Map.GetPlayerMapPosition(loc, "player")
    local coords = math.ceil(pos.x * 10000)/100 .. " " .. math.ceil(pos.y * 10000)/100
    return coords
end

function RareUtil_GetUnitIDFromGUID(UnitGUID)
    local v1, v2, v3, v4, v5, v6 = strsplit("-", UnitGUID)
    return tonumber(v6)
end

function RareUtil_GetTargetHealth()
    local currentHealth = UnitHealth("target")
    local maxHealth = UnitHealthMax("target")
    local healthPercentage = math.floor(currentHealth / maxHealth * 100)
    return healthPercentage
end

function RareUtil_GetRespawnTimer(zone, UnitID)
    local respawnTimer = nil
    for iterationZone, rares in pairs(raresInfo) do
        for i = 1, #rares do
            local raresI = rares[i]
            if UnitID == raresI[1] then
                local raresI9 = raresI[10]
                if raresI9[1] == 0 and raresI9[2] == 0 then
                    return respawnTimer
                elseif raresI9[1] == 0 and raresI9[2] ~= 0 then
                    return respawnTimer
                elseif raresI9[1] ~= 0 and raresI9[2] == 0 then
                    return respawnTimer
                elseif raresI9[1] ~= 0 and raresI9[2] ~= 0 then
                    if raresI9[1] == raresI9[2] then
                        respawnTimer = (RareUtil_CalculateTimer(raresI9[1]) .. " (Beta Function!)")
                        return respawnTimer
                    else
                        respawnTimer = (RareUtil_CalculateTimer(raresI9[1]) .. dash .. RareUtil_CalculateTimer(raresI9[2]) .. " (Beta Function!)")
                        return respawnTimer
                    end
                end
            end
        end
    end
end

function RareUtil_CalculateTimer(RareTimer)
    local CalculatedTimer = nil
    local curH, curM = GetGameTime()
    local totalTimerM = nil
    local respawmTimerM = nil
    local respawmTimerH = nil

    totalTimer = curH * 60 + curM + RareTimer
    totalTimerH = math.floor(totalTimer / 60)
    totalTimerM = totalTimer % 60
    CalculatedTimer = tostring(totalTimerH) .. ":" .. tostring(totalTimerM)
    return CalculatedTimer
end

function RareUtil_ToggleAuto()
    if RareUtil_AutomaticAnnounce == false then
        RareUtil_AutomaticAnnounce = true
        AutomaticOptionsCheckButton:SetChecked(true)
        ChatFrame1:AddMessage(L["Automatic Mode is Enabled!"])
    elseif RareUtil_AutomaticAnnounce == true then
        RareUtil_AutomaticAnnounce = false
        AutomaticOptionsCheckButton:SetChecked(false)
        ChatFrame1:AddMessage(L["Automatic Mode is Disabled!"])
    end
end

function RareUtil_PrintHelp()
    ChatFrame1:AddMessage("/rareutil help;" .. L["display this message"])
    ChatFrame1:AddMessage("/rareutil options;" .. L["opens the options panel"])
    ChatFrame1:AddMessage("/rareutil frame;" .. L["toggles the Addon Frame"])
    ChatFrame1:AddMessage("/rareutil icon;" .. L["toggles the minimap button/icon"])
    ChatFrame1:AddMessage("/rareutil toggle;" .. L["Toggles the Automatic Announcement function on or off."])
end

function RareUtil_OpenOptions()
    InterfaceOptionsFrame_OpenToCategory("AzerPUG RareUtility")
    InterfaceOptionsFrame_OpenToCategory("AzerPUG RareUtility")
end

function RareUtil_AddonFrameToggle()
    if RareUtilAddonFrame:IsShown() then
        RareUtilAddonFrame:Hide()
    elseif not RareUtilAddonFrame:IsShown() then
        RareUtilAddonFrame:Show()
    end
end

SLASH_RAREUTIL1 = "/rareutil"
SlashCmdList["RAREUTIL"] = function(msg, editbox)
    if string.find(msg, "options") then
        RareUtil_OpenOptions()
    elseif string.find(msg, "frame") then
        RareUtil_AddonFrameToggle()
    elseif string.find(msg, "toggle") then
        RareUtil_ToggleAuto()
    elseif string.find(msg, "help") then
        RareUtil_PrintHelp()
    end
end

RareUtil_OnLoad(self)